/* --COPYRIGHT--,BSD_EX
 * Copyright (c) 2019, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *******************************************************************************
 *
 *                       MSP430 CODE EXAMPLE DISCLAIMER
 *
 * MSP430 code examples are self-contained low-level programs that typically
 * demonstrate a single peripheral function or device feature in a highly
 * concise manner. For this the code may rely on the device's power-on default
 * register values and settings such as the clock configuration and care must
 * be taken when combining code from several examples to avoid potential side
 * effects. Also see www.ti.com/grace for a GUI- and www.ti.com/msp430ware
 * for an API functional library-approach to peripheral configuration.
 *
 * --/COPYRIGHT--*/
//******************************************************************************
//  MSP430FR235x Demo: Half-Wave Rectifier Circuit
//
//  Description: Configure SAC0 in DAC mode to internally provide the bias
//  voltage for the non-inverting op-amp input. Connect the appropriate external
//  circuitry to realize the half-wave rectifier circuit. SAC is configured to
//  low power mode.
//  ACLK = n/a, MCLK = SMCLK = default DCODIV ~1MHz.
//
//                MSP430FR235x
//             -------------------
//         /|\|          P1.2/OA0-|----||----------------||--R1=1Kohm<---Vin
//          | |                   |    ||                ||
//          | |                   |  SD2(Diode)       Rf=1Kohm
//          --|RST                |    ||                ||
//            |       P1.1/OA0O/A1|----||---SD1(Diode)---||----->Vout
//            |                   |
//            |          P1.3/OA0+|*Vbias set by internal DAC*
//            |                   |
//
//   Matt Calvo
//   Texas Instruments Inc.
//   December 2019
//   Built with IAR Embedded Workbench v7.12.1 & Code Composer Studio v9.0.1
//******************************************************************************
#include <msp430.h>

int main(void)
{
    WDTCTL = WDTPW + WDTHOLD;                 // Stop watch dog timer

    // Disable the GPIO power-on default high-impedance mode
    // to activate previously configured port settings
    PM5CTL0 &= ~LOCKLPM5;

    // Configure reference module
    PMMCTL0_H = PMMPW_H;                      // Unlock the PMM registers
    PMMCTL2 = INTREFEN | REFVSEL_2;           // Enable internal 2.5V reference
    while(!(PMMCTL2 & REFGENRDY));            // Poll till internal reference settles

    // Configure SAC0 in DAC mode to supply Vbias and
    // realize the half-wave rectifier circuit
    P1SEL0 |= BIT1 + BIT2;                    // Select P1.1(OA0O) and P1.2(OA0-)
                                              // as OA function
    P1SEL1 |= BIT1 + BIT2;                    // Select P1.1(OA0O) and P1.2(OA0-)
                                              // as OA function

    SAC0DAT = 0xA8F;                          // Set SAC0 DAC output = 1.65V
    // With a 12-bit DAC: 1.65V/2.5V = 0.66
    // 0.66 * 2^12 = 2703 = 0xA8F

    SAC0DAC |= DACSREF_1 | DACEN;             // Select internal shared reference
                                              // and enable SAC0 DAC
    SAC0OA |= NMUXEN + PMUXEN + NSEL_0 + PSEL_1; // Enable SAC0 negative and
                                                 // positive input in DAC mode
    SAC0OA |= OAPM;                           // Select low speed and low power mode
    SAC0OA |= SACEN + OAEN;                   // Enable SAC0 and OA

    __bis_SR_register(LPM3_bits);             // Enter LPM3

}